@@ -30,6 +30,10 @@
 #include <math.h>
 #include <string.h>
 
+// it's so ugly --hb ;(
+// include before, cause it has extern "C" already
+
+#if 1
 extern "C" {
 
 #include "paginate_gdiprint.h"
@@ -37,14 +41,17 @@ extern "C" {
 #include "diagram.h"
 #include "diagramdata.h"
 
-#include "render.h"
 #include "filter.h"
 
 #include "message.h"
 
 }
+#endif
 
 namespace W32 {
+// can't 
+// #define WIN32_LEAN_AND_MEAN
+// because we need stuff like PRINTDLG
 #include <windows.h>
 }
 
@@ -72,9 +79,9 @@ print_page(DiagramData *data, DiaExportFilter* pExp, W32::HANDLE hDC,
   /* set up clip mask ? */
 
   /* render the region */
-  W32::StartPage(hDC);
-  pExp->export(&page_data, "", "", hDC);
-  W32::EndPage(hDC);
+  W32::StartPage((W32::HDC)hDC);
+  pExp->export_func(&page_data, "", "", (W32::HDC)hDC);
+  W32::EndPage((W32::HDC)hDC);
 
   return nobjs;
 }
@@ -103,10 +110,16 @@ paginate_gdiprint(Diagram *dia, DiaExportFilter* pExp, W32::HANDLE hDC)
   }
 
   /* iterate through all the pages in the diagram */
-  for (y = inity, ypos = 0; y < extents->bottom; y += height, ypos++)
-    for (x = inity, xpos = 0; x < extents->right; x += width, xpos++) {
+  for (y = inity, ypos = 0; y < extents->bottom; y += height, ypos++) {
+    /* ensure we are not producing pages for epsilon */
+    if ((extents->bottom - y) < 1e-6)
+      break;
+    for (x = initx, xpos = 0; x < extents->right; x += width, xpos++) {
       Rectangle page_bounds;
 
+      if ((extents->right - x) < 1e-6)
+	break;
+
       page_bounds.left = x;
       page_bounds.right = x + width;
       page_bounds.top = y;
@@ -114,7 +127,7 @@ paginate_gdiprint(Diagram *dia, DiaExportFilter* pExp, W32::HANDLE hDC)
 
       nobjs += print_page(dia->data, pExp, hDC, &page_bounds, xpos, ypos);
     }
-
+  }
 }
 
 /* to remember changes made in the print dialog, finally leaked. */
@@ -127,10 +140,11 @@ diagram_print_gdi(Diagram *dia)
 {
   W32::PRINTDLG printDlg;
   W32::DOCINFO  docInfo;
+  W32::DEVMODE* pDevMode;
   DiaExportFilter* pExp = NULL;
   int i;
 
-  pExp = filter_guess_export_filter("dummy.wmf");
+  pExp = filter_get_by_name("wmf");
 
   if (!pExp) {
     message_error("Can't print without the WMF plugin installed");
@@ -142,15 +156,51 @@ diagram_print_gdi(Diagram *dia)
   printDlg.hDevNames = hDevNames;
 
   printDlg.Flags = 0;
-  printDlg.Flags |= PD_RETURNDC | PD_NOSELECTION;
   printDlg.nMinPage = printDlg.nMaxPage = 0;
   printDlg.nCopies = 1;
   printDlg.lStructSize = sizeof (W32::PRINTDLG);
 
+  /* Uhmm, first call to initialize device settings ... */
+  if (!printDlg.hDevMode) {
+    printDlg.Flags = PD_RETURNDEFAULT;
+    if (!W32::PrintDlg (&printDlg))
+      g_warning ("Failed to get printer defaults.");
+  }
+
+  pDevMode = (W32::DEVMODE*) W32::GlobalLock (printDlg.hDevMode);
+  if (pDevMode) {
+    /* initialize with Dia default */
+    pDevMode->dmFields |= DM_ORIENTATION;
+    pDevMode->dmOrientation = dia->data->paper.is_portrait ?
+      DMORIENT_PORTRAIT : DMORIENT_LANDSCAPE;
+
+    /* Maybe we could adjust the scaling here as well but are al of:
+     *   dmPaperSize, dmPaperLength, dmPaperWidth, dmScale
+     * initialized despite of the api documentation ?
+     */
+    g_print ("Paper size %d, length %d width %d scale %d\n",
+             pDevMode->dmPaperSize, 
+             pDevMode->dmPaperLength, pDevMode->dmPaperWidth, pDevMode->dmScale);
+
+    W32::GlobalUnlock (printDlg.hDevMode);
+  }
+
+  printDlg.Flags = PD_RETURNDC | PD_NOSELECTION;
+
   if (!W32::PrintDlg (&printDlg)) {
     W32::DWORD dwError = W32::CommDlgExtendedError ();
-    if (dwError)
-      message_error("Print Dialog failed with error %d", dwError);
+    if (dwError) {
+	const gchar *emsg;
+      /* ugly common dialog api does *not* use standard windoze error codes */
+	if (dwError < PDERR_PRINTERCODES) emsg = "Common Dialog Error";
+	else if (dwError == PDERR_LOADDRVFAILURE) emsg = "Failed to load driver";
+	else if (dwError == PDERR_PRINTERNOTFOUND) emsg = "Printer Not Found";
+	else if (dwError < CFERR_CHOOSEFONTCODES) emsg = "Printer Selection Error";
+	else emsg = "Unexpected Error"; /* Other common dialogs errors */
+
+      message_error("Print Dialog failed with error %d\n%s", 
+                    dwError, emsg);
+    }
     return;
   }
 
