@@ -55,6 +55,8 @@
 #include <locale.h>
 #include <errno.h>
 
+#include <glib/gstdio.h>
+
 #include "intl.h"
 #include "render_eps.h"
 #include "message.h"
@@ -98,16 +100,21 @@ export_render_eps(DiaPsRenderer *renderer,
 {
   FILE *outfile;
 
-  outfile = fopen(filename, "w");
+  outfile = g_fopen(filename, "w");
   if (outfile == NULL) {
-    message_error(_("Can't open output file %s: %s\n"), filename, strerror(errno));
+    message_error(_("Can't open output file %s: %s\n"), 
+		  dia_message_filename(filename), strerror(errno));
     g_object_unref(renderer);
     return;
   }
   renderer->file = outfile;
   renderer->scale = 28.346 * data->paper.scaling;
   renderer->extent = data->extents;
-  renderer->is_eps = TRUE;
+  renderer->pstype = (guint)user_data;
+  if (renderer->pstype & PSTYPE_EPSI) {
+    /* Must store the diagram for making a preview */
+    renderer->diagram = data;
+  }
 
   if (renderer->file) {
     renderer->title = g_strdup (diafilename);
@@ -129,27 +136,48 @@ new_psprint_renderer(Diagram *dia, FILE *file)
   renderer = g_object_new (DIA_TYPE_PS_RENDERER, NULL);
 #endif
   renderer->file = file;
-  renderer->is_eps = FALSE;
+  renderer->pstype = PSTYPE_PS;;
 
   return DIA_RENDERER(renderer);
 }
 
+static const gchar *eps_extensions[] = { "eps", NULL };
+#if 0
+static const gchar *epsi_extensions[] = { "epsi", NULL };
+#endif
 #ifdef HAVE_FREETYPE
-static const gchar *extensions[] = { "eps", "epsi", NULL };
 DiaExportFilter eps_ft2_export_filter = {
   N_("Encapsulated Postscript (using Pango fonts)"),
-  extensions,
+  eps_extensions,
   export_ft2_eps,
-  NULL,
+  GINT_TO_POINTER(PSTYPE_EPS), /* user_data */
   "eps-pango"
 };
+/* Disabled until we can actually make the preview. */
+#  if 0
+DiaExportFilter epsi_ft2_export_filter = {
+  N_("Encapsulated Postscript with preview (using Pango fonts)"),
+  epsi_extensions,
+  export_ft2_eps,
+  GINT_TO_POINTER(PSTYPE_EPSI), /* user_data */
+  "epsi-pango"
+};
+#  endif
 #endif
 
 DiaExportFilter eps_export_filter = {
   N_("Encapsulated Postscript (using PostScript Latin-1 fonts)"),
-  extensions,
+  eps_extensions,
   export_eps,
-  NULL,
+  GINT_TO_POINTER(PSTYPE_EPS), /* user_data */
   "eps-builtin"
 };
-
+/* Commented out until we can actually make the preview.
+DiaExportFilter epsi_export_filter = {
+  N_("Encapsulated Postscript with preview (using PostScript Latin-1 fonts)"),
+  epsi_extensions,
+  export_eps,
+  PSTYPE_EPSI,
+  "epsi-builtin"
+};
+*/
