@@ -34,6 +34,8 @@
 #include "../lib/plug-ins.h"
 #include "../lib/sheet.h"
 #include "../lib/message.h"
+#include "../lib/object.h"
+#include "../app/pixmaps/missing.xpm"
 
 #include "interface.h"
 #include "sheets.h"
@@ -55,7 +57,7 @@ SheetObjectMod *
 sheets_append_sheet_object_mod(SheetObject *so, SheetMod *sm)
 {
   SheetObjectMod *sheet_object_mod;
-  ObjectType *ot;
+  DiaObjectType *ot;
 
   sheet_object_mod = g_new(SheetObjectMod, 1);
   sheet_object_mod->sheet_object = *so;
@@ -63,7 +65,7 @@ sheets_append_sheet_object_mod(SheetObject *so, SheetMod *sm)
 
   ot = object_get_type(so->object_type);
   g_assert(ot);
-  if (ot->ops == ((ObjectType *)(custom_type_symbol))->ops)
+  if (ot->ops == ((DiaObjectType *)(custom_type_symbol))->ops)
     sheet_object_mod->type = OBJECT_TYPE_SVG;
   else
     sheet_object_mod->type = OBJECT_TYPE_PROGRAMMED;
@@ -146,7 +148,7 @@ sheets_optionmenu_create(GtkWidget *option_menu, GtkWidget *wrapbox,
       continue;
 
     {
-      menu_item = gtk_menu_item_new_with_label(sheet_mod->sheet.name);
+      menu_item = gtk_menu_item_new_with_label(gettext(sheet_mod->sheet.name));
 
       gtk_menu_append(GTK_MENU(optionmenu_menu), menu_item);
       
@@ -338,8 +340,14 @@ create_object_pixmap(SheetObject *so, GtkWidget *parent,
         gdk_pixbuf_render_pixmap_and_mask(pixbuf, pixmap, mask, 1.0);
         gdk_pixbuf_unref(pixbuf);
       } else {
-        g_warning (error->message);
+        message_warning ("%s", error->message);
         g_error_free (error);
+	*pixmap = gdk_pixmap_colormap_create_from_xpm_d
+	  (NULL,
+	   gtk_widget_get_colormap(parent),
+	   mask, 
+	   &style->bg[GTK_STATE_NORMAL],
+	   missing);
       }
     }
     else
@@ -447,10 +455,10 @@ sheet_object_mod_get_type_string(SheetObjectMod *som)
   case OBJECT_TYPE_SVG:
     return _("SVG Shape");
   case OBJECT_TYPE_PROGRAMMED:
-    return _("Programmed Object");
+    return _("Programmed DiaObject");
   default:
     g_assert_not_reached();
-    return FALSE;           /* shut gcc up */
+    return "";
   }
 }
 
