@@ -22,6 +22,7 @@
 #include "diatypes.h"
 #include "geometry.h"
 #include "color.h"
+#include "dia_xml.h"
 
 /* NOTE: Add new arrow types at the end, or the enums
    will change order leading to file incompatibilities. */
@@ -53,13 +54,38 @@ typedef enum {
   ARROW_CROSS,                   /* Vertical line */
   ARROW_FILLED_CONCAVE,
   ARROW_BLANKED_CONCAVE,
+  ARROW_ROUNDED,
+  ARROW_HALF_DIAMOND,		/* ---< */
+  ARROW_OPEN_ROUNDED,		/* ---c */
+  ARROW_FILLED_DOT_N_TRIANGLE,	/* ---|>o */
+  ARROW_ONE_OR_MANY,            /* ER-model: 1 or many*/
+  ARROW_NONE_OR_MANY,           /* ER-model: 0 or many*/
+  ARROW_ONE_OR_NONE,            /* ER-model: 1 or 0 */
+  ARROW_ONE_EXACTLY,            /* ER-model: exactly one*/
+  ARROW_BACKSLASH,                  /* -\----  */
+  ARROW_THREE_DOTS,
+
+  MAX_ARROW_TYPE /* No arrow heads may be defined beyond here. */
 } ArrowType;
 
 struct menudesc {
   char *name;
-  int enum_value;
+  ArrowType enum_value;
 };
 
+/** The number of centimeters long and wide an arrow starts with by default.
+ * This can be changed without breaking old diagrams, as the arrow width
+ * is stored in there.
+ * Note:  Currently, many places have this number hardcoded.
+ * find . -name \*.[ch] | xargs grep \\.8
+ */
+#define DEFAULT_ARROW_SIZE 0.5
+
+/** The minimum width or length of an arrowhead.  This to avoid borderline
+ *  cases that break trig functions, as seen in bug #144394
+ */
+#define MIN_ARROW_DIMENSION 0.001
+
 /* These are used to fill menus.  See dia_arrow_fill_menu in widgets.c */
 DIAVAR struct menudesc arrow_types[];
 
@@ -69,6 +95,7 @@ struct _Arrow {
   real width;
 };
 
+
 void arrow_draw(DiaRenderer *renderer, ArrowType type,
 		Point *to, Point *from,
 		real length, real width, real linewidth,
@@ -77,12 +104,24 @@ void arrow_draw(DiaRenderer *renderer, ArrowType type,
 void
 calculate_arrow_point(const Arrow *arrow, const Point *to, const Point *from,
 		      Point *move_arrow, Point *move_line,
-		      const real linewidth);
+		      real linewidth);
 
 /* Transforms 'start' to be at the back end of the arrow, and puts the
  * tip of the arrow into 'arrowtip'.
  */
 void arrow_transform_points(Arrow *arrow, Point *start, Point *to,
-                       int linewidth, Point *arrowtip);
+			    int linewidth, Point *arrowtip);
+
+void save_arrow(ObjectNode obj_node, Arrow *arrow, gchar *type_attribute,
+		gchar *length_attribute, gchar *width_attribute);
+void load_arrow(ObjectNode obj_node, Arrow *arrow, gchar *type_attribute, 
+		gchar *length_attribute, gchar *width_attribute);
+
+/** Returns the ArrowType for a given name of an arrow, or 0 if not found. */
+ArrowType arrow_type_from_name(gchar *name);
+/** Returns the index in arrow_types of the given arrow type. */
+gint arrow_index_from_type(ArrowType type);
+gchar *arrow_get_name_from_type(ArrowType type);
+GList *get_arrow_names(void);
 
 #endif /* ARROWS_H */
