@@ -24,6 +24,7 @@
 #include "diarenderer.h"
 #include "object.h"
 #include "text.h"
+#include "textline.h"
 
 struct _BezierApprox {
   Point *points;
@@ -93,6 +94,8 @@ static void draw_image (DiaRenderer *renderer,
                         DiaImage image);
 static void draw_text  (DiaRenderer *renderer,
                         Text *text);
+static void draw_text_line  (DiaRenderer *renderer,
+			     TextLine *text_line, Point *pos, Color *color);
 
 static void draw_rect (DiaRenderer *renderer,
                        Point *ul_corner, Point *lr_corner,
@@ -249,7 +252,8 @@ dia_renderer_class_init (DiaRendererClass *klass)
   renderer_class->draw_rounded_polyline  = draw_rounded_polyline;
   renderer_class->draw_polyline  = draw_polyline;
   renderer_class->draw_polygon   = draw_polygon;
-  renderer_class->draw_text    = draw_text;
+  renderer_class->draw_text      = draw_text;
+  renderer_class->draw_text_line = draw_text_line;
 
   /* highest level functions */
   renderer_class->draw_rounded_rect = draw_rounded_rect;
@@ -320,9 +324,11 @@ set_fillstyle (DiaRenderer *renderer, FillStyle mode)
 static void 
 set_font (DiaRenderer *renderer, DiaFont *font, real height)
 {
+  /* if it's the same font we must ref it first */
+  dia_font_ref (font);
   if (renderer->font)
     dia_font_unref (renderer->font);
-  renderer->font = dia_font_ref (font);
+  renderer->font = font;
   renderer->font_height = height;
 }
 
@@ -399,20 +405,36 @@ static void draw_text(DiaRenderer *renderer,
 		      Text *text) {
   Point pos;
   int i;
-  
-  DIA_RENDERER_GET_CLASS(renderer)->set_font(renderer, text->font, text->height);
-  
+
   pos = text->position;
   
   for (i=0;i<text->numlines;i++) {
-    DIA_RENDERER_GET_CLASS(renderer)->draw_string(renderer,
-						  text->line[i],
-						  &pos, text->alignment,
-						  &text->color);
+    Point aligned_pos = pos;
+    TextLine *text_line = text->lines[i];
+
+    aligned_pos.x -= text_line_get_alignment_adjustment(text_line,
+							text->alignment);
+    DIA_RENDERER_GET_CLASS(renderer)->draw_text_line(renderer,
+						     text_line,
+						     &aligned_pos,
+						     &text->color);
     pos.y += text->height;
   }
 }
 
+/** Default implementation of draw_text_line */
+static void draw_text_line(DiaRenderer *renderer,
+			   TextLine *text_line, Point *pos, Color *color) {
+  DIA_RENDERER_GET_CLASS(renderer)->set_font(renderer, 
+					     text_line_get_font(text_line),
+					     text_line_get_height(text_line));
+  
+  DIA_RENDERER_GET_CLASS(renderer)->draw_string(renderer,
+						text_line_get_string(text_line),
+						pos, ALIGN_LEFT,
+						color);
+}
+
 static void 
 draw_image (DiaRenderer *renderer,
             Point *point, real width, real height,
@@ -434,7 +456,7 @@ draw_image (DiaRenderer *renderer,
  * [ 1  0  0  0]
  * (At least that's what Hearn and Baker says for beziers.)
  */
-#define BEZIER_SUBDIVIDE_LIMIT 0.03
+#define BEZIER_SUBDIVIDE_LIMIT 0.01
 #define BEZIER_SUBDIVIDE_LIMIT_SQ (BEZIER_SUBDIVIDE_LIMIT*BEZIER_SUBDIVIDE_LIMIT)
 
 static void
@@ -673,7 +695,7 @@ draw_polyline (DiaRenderer *renderer,
        (ie, angle between lines p1,p2 and p2,p3)
      then maxr = c * sin(a/2)
  */
-real
+static real
 calculate_min_radius( Point *p1, Point *p2, Point *p3 )
 {
   real c;
@@ -950,8 +972,10 @@ draw_polyline_with_arrows(DiaRenderer *renderer,
     point_sub(&points[lastline-1], &move_line);
   }
   /* Don't draw degenerate line segments at end of line */
-  DIA_RENDERER_GET_CLASS(renderer)->draw_polyline(renderer, &points[firstline], 
-			       lastline-firstline, color);
+  if (lastline-firstline > 1) /* probably hiding a bug above, but don't try to draw a negative
+                                                                  * number of points at all, fixes bug #148139 */
+    DIA_RENDERER_GET_CLASS(renderer)->draw_polyline(renderer, &points[firstline], 
+                                                    lastline-firstline, color);
   if (start_arrow != NULL && start_arrow->type != ARROW_NONE)
     arrow_draw(renderer, start_arrow->type,
 	       &start_arrow_head, &points[firstline+1],
@@ -1021,10 +1045,11 @@ draw_rounded_polyline_with_arrows(DiaRenderer *renderer,
     point_sub(&points[lastline-1], &move_line);
   }
   /* Don't draw degenerate line segments at end of line */
-  DIA_RENDERER_GET_CLASS(renderer)->draw_rounded_polyline(renderer,
-                                                          &points[firstline],
-                                                          lastline-firstline,
-                                                          color, radius);
+  if (lastline-firstline > 1) /* only if there is something */
+    DIA_RENDERER_GET_CLASS(renderer)->draw_rounded_polyline(renderer,
+                                                            &points[firstline],
+                                                            lastline-firstline,
+                                                            color, radius);
   if (start_arrow != NULL && start_arrow->type != ARROW_NONE)
     arrow_draw(renderer, start_arrow->type,
 	       &start_arrow_head, &points[firstline+1],
@@ -1144,16 +1169,42 @@ find_center_point(Point *center, Point *p1, Point *p2, Point *p3)
   /* The intersection between these two is the center */
   if (!intersection_line_line(center, &mid1, &orth1, &mid2, &orth2)) {
     /* Degenerate circle */
-    /* Either the points are really close together, or directly apart */
+    /* Case 1: Points are all on top of each other.  Nothing to do. */
     if (fabs((p1->x + p2->x + p3->x)/3 - p1->x) < 0.0000001 &&
-	fabs((p1->y + p2->y + p3->y)/3 - p1->y) < 0.0000001)
+	fabs((p1->y + p2->y + p3->y)/3 - p1->y) < 0.0000001) {
       return FALSE;
+    }
     
-    return TRUE;
+    /* Case 2: Two points are on top of each other.  Midpoint of
+     * non-degenerate line is center. */
+    if (distance_point_point_manhattan(p1, p2) < 0.0000001) {
+      *center = mid2;
+      return TRUE;
+    } else if (distance_point_point_manhattan(p1, p3) < 0.0000001 ||
+	       distance_point_point_manhattan(p2, p3) < 0.0000001) {
+      *center = mid1;
+      return TRUE;
+    }
+    /* Case 3: All points on a line.  Nothing to do. */
+    return FALSE;
   }
   return TRUE;
 }
 
+static gboolean
+is_right_hand (const Point *a, const Point *b, const Point *c)
+{
+  Point dot1, dot2;
+
+  dot1 = *a;
+  point_sub(&dot1, c);
+  point_normalize(&dot1);
+  dot2 = *b;
+  point_sub(&dot2, c);
+  point_normalize(&dot2);
+  return point_cross(&dot1, &dot2) > 0;
+}
+
 static void
 draw_arc_with_arrows (DiaRenderer *renderer, 
                       Point *startpoint, 
@@ -1167,8 +1218,7 @@ draw_arc_with_arrows (DiaRenderer *renderer,
   Point oldstart = *startpoint;
   Point oldend = *endpoint;
   Point center;
-  real width, angle1, angle2;
-  Point dot1, dot2;
+  real width, angle1, angle2, arrow_ofs = 0.0;
   gboolean righthand;
   Point start_arrow_head;
   Point start_arrow_end;
@@ -1177,17 +1227,13 @@ draw_arc_with_arrows (DiaRenderer *renderer,
 
   if (!find_center_point(&center, startpoint, endpoint, midpoint)) {
     /* Degenerate circle -- should have been caught by the drawer? */
+    printf("Degenerate\n");
   }
 
-
-  dot1 = *startpoint;
-  point_sub(&dot1, endpoint);
-  point_normalize(&dot1);
-  dot2 = *midpoint;
-  point_sub(&dot2, endpoint);
-  point_normalize(&dot2);
-  righthand = point_cross(&dot1, &dot2) > 0;
+  righthand = is_right_hand (startpoint, midpoint, endpoint);
   
+  width = 2*distance_point_point(&center, startpoint);
+
   if (start_arrow != NULL && start_arrow->type != ARROW_NONE) {
     Point move_arrow, move_line;
     real tmp;
@@ -1210,6 +1256,7 @@ draw_arc_with_arrows (DiaRenderer *renderer,
     start_arrow_head = *startpoint;
     point_sub(&start_arrow_head, &move_arrow);
     point_sub(startpoint, &move_line);
+    arrow_ofs += sqrt (move_line.x * move_line.x + move_line.y * move_line.y);
   }
   if (end_arrow != NULL && end_arrow->type != ARROW_NONE) {
     Point move_arrow, move_line;
@@ -1233,27 +1280,41 @@ draw_arc_with_arrows (DiaRenderer *renderer,
     end_arrow_head = *endpoint;
     point_sub(&end_arrow_head, &move_arrow);
     point_sub(endpoint, &move_line);
+    arrow_ofs += sqrt (move_line.x * move_line.x + move_line.y * move_line.y);
   }
 
-  if (!find_center_point(&center, startpoint, endpoint, midpoint)) {
-    /* Not sure what to do here */
-    *startpoint = oldstart;
-    *endpoint = oldend;
-    return;
-  }
-  width = 2*distance_point_point(&center, startpoint);
-  angle1 = -atan2(startpoint->y-center.y, startpoint->x-center.x)*180.0/G_PI;
+  /* Now we possibly have new start- and endpoint. We must not
+   * recalculate the center cause the new points lie on the tangential
+   * approximation of the original arc arrow lines not on the arc itself. 
+   * The one thing we need to deal with is calculating the (new) angles 
+   * and get rid of the arc drawing altogether if got degenerated.
+   *
+   * Why shouldn't we recalculate the whole thing from the new start/endpoints?
+   * Done this way the arc does not come out the back of the arrows.
+   *  -LC, 20/2/2006
+   */
+  angle1 = -atan2(startpoint->y - center.y, startpoint->x - center.x)*180.0/G_PI;
   while (angle1 < 0.0) angle1 += 360.0;
-  angle2 = -atan2(endpoint->y-center.y, endpoint->x-center.x)*180.0/G_PI;
+  angle2 = -atan2(endpoint->y - center.y, endpoint->x - center.x)*180.0/G_PI;
   while (angle2 < 0.0) angle2 += 360.0;
   if (righthand) {
     real tmp = angle1;
     angle1 = angle2;
     angle2 = tmp;
   }
+  /* now with the angles we can bring the startpoint back to the arc, but there must be a less expensive way to do this? */
+  if (start_arrow != NULL && start_arrow->type != ARROW_NONE) {
+    startpoint->x = cos (G_PI * angle1 / 180.0) * width / 2.0 + center.x;
+    startpoint->y = sin (G_PI * angle1 / 180.0) * width / 2.0 + center.y;
+  }
+  if (end_arrow != NULL && end_arrow->type != ARROW_NONE) {
+    endpoint->x = cos (G_PI * angle1 / 180.0) * width / 2.0 + center.x;
+    endpoint->y = sin (G_PI * angle1 / 180.0) * width / 2.0 + center.y;
+  }
 
   DIA_RENDERER_GET_CLASS(renderer)->draw_arc(renderer, &center, width, width,
-			  angle1, angle2, color);
+					     angle1, angle2, color);
+
   if (start_arrow != NULL && start_arrow->type != ARROW_NONE)
     arrow_draw(renderer, start_arrow->type,
 	       &start_arrow_head, &start_arrow_end,
