@@ -44,11 +44,11 @@
 #include "arrows.h"
 #include "color.h"
 #include "font.h"
-#include "object.h"
 #include "dia_xml.h"
 #include "intl.h"
 #include "textattr.h"
 
+G_BEGIN_DECLS
 
 typedef gboolean (*PropDescToPropPredicate)(const PropDescription *pdesc);
 
@@ -64,9 +64,9 @@ struct _PropDialog { /* This is to be treated as opaque ! */
   GArray *prop_widgets; /* of PropWidgetAssoc. This is a "flat" listing of 
                            properties and widgets (no nesting information is 
                            kept here) */
-  Object *obj_copy; /* if !NULL, a copy of the object which can be used 
+  DiaObject *obj_copy; /* if !NULL, a copy of the object which can be used 
                        as scratch. */
-  Object *orig_obj; /* The original object (do not touch !) */
+  DiaObject *orig_obj; /* The original object (do not touch !) */
 
   GPtrArray *containers;
   WIDGET *lastcont;
@@ -81,7 +81,7 @@ struct _PropEventData {
   Property *self; 
 };
 
-typedef gboolean (*PropEventHandler) (Object *obj, Property *prop);
+typedef gboolean (*PropEventHandler) (DiaObject *obj, Property *prop);
 
 struct _PropEventHandlerChain {
   PropEventHandler handler;
@@ -162,6 +162,8 @@ typedef const gchar *PropertyType;
 
 /* Geometric types : */
 #define PROP_TYPE_REAL "real"                /* RealProperty */
+#define PROP_TYPE_LENGTH "length"            /* LengthProperty */
+#define PROP_TYPE_FONTSIZE "fontsize"        /* FontsizeProperty */
 #define PROP_TYPE_POINT "point"              /* PointProperty */
 #define PROP_TYPE_POINTARRAY "pointarray"    /* PointarrayProperty */
 #define PROP_TYPE_BEZPOINT "bezpoint"        /* BezPointProperty */
@@ -254,7 +256,8 @@ typedef void (*FreeRecordFunc)(gpointer rec);
 struct _PropDescCommonArrayExtra { /* don't use this directly. 
                                       Use one of below */
   PropDescription *record;
-  const gchar *composite_type; /* can be NULL. */ 
+  PropOffset      *offsets; /* the offsets into the structs in the list/array */
+  const gchar     *composite_type; /* can be NULL. */ 
 };
 
 struct _PropDescDArrayExtra {
@@ -399,45 +402,45 @@ struct _PropOffset {
 
 /* returns TRUE if this object can be handled (at least in part) through this
    library. */
-gboolean object_complies_with_stdprop(const Object *obj);
+gboolean object_complies_with_stdprop(const DiaObject *obj);
 
 /* will do whatever is needed to make the PropDescription * list look good to 
    the rest of the properties code. Can return NULL. */
-const PropDescription *object_get_prop_descriptions(const Object *obj);
+const PropDescription *object_get_prop_descriptions(const DiaObject *obj);
 
-gboolean object_get_props_from_offsets(Object *obj, PropOffset *offsets,
+gboolean object_get_props_from_offsets(DiaObject *obj, PropOffset *offsets,
 				       GPtrArray *props);
-gboolean object_set_props_from_offsets(Object *obj, PropOffset *offsets,
+gboolean object_set_props_from_offsets(DiaObject *obj, PropOffset *offsets,
 				       GPtrArray *props);
 
 /* apply some properties and return a corresponding object change */
-ObjectChange *object_apply_props(Object *obj, GPtrArray *props);
+ObjectChange *object_apply_props(DiaObject *obj, GPtrArray *props);
 
 /* standard properties dialogs that can be used for objects that
  * implement describe_props, get_props and set_props.
  * If is_default is set, this is a default dialog, not an object dialog.
  */
-WIDGET *object_create_props_dialog     (Object *obj, gboolean is_default);
-ObjectChange *object_apply_props_from_dialog (Object *obj, WIDGET *dialog);
+WIDGET *object_create_props_dialog     (DiaObject *obj, gboolean is_default);
+ObjectChange *object_apply_props_from_dialog (DiaObject *obj, WIDGET *dialog);
 
 /* create a property from the object's property descriptors. To be freed with
    prop->ops->free(prop); or put it into a single property list. NULL if object
    has nothing matching. Property's value is initialised by the object.
    Serve cold. */
-Property *object_prop_by_name(Object *obj, const char *name);
-Property *object_prop_by_name_type(Object *obj, const char *name, const char *type);
+Property *object_prop_by_name(DiaObject *obj, const char *name);
+Property *object_prop_by_name_type(DiaObject *obj, const char *name, const char *type);
 
 /* standard way to load/save properties of an object */
-void          object_load_props(Object *obj, ObjectNode obj_node);
-void          object_save_props(Object *obj, ObjectNode obj_node);
+void          object_load_props(DiaObject *obj, ObjectNode obj_node);
+void          object_save_props(DiaObject *obj, ObjectNode obj_node);
 
 /* standard way to copy the properties of an object into another (of the
    same type) */
-void          object_copy_props(Object *dest, const Object *src,
+void          object_copy_props(DiaObject *dest, const DiaObject *src,
                                 gboolean is_default);
 
 /* Return a reference to objects property with 'name' or NULL */
-Property     *object_get_prop_by_name (Object *obj, const char* name);
+Property     *object_get_prop_by_name (DiaObject *obj, const char* name);
  
 /* ************************************************************* */ 
 
@@ -460,8 +463,8 @@ void stdprops_init(void);
  * on Win32
  */
 #ifdef G_OS_WIN32
-static PropNumData prop_std_line_width_data = { 0.0, 10.0, 0.01 };
-static PropNumData prop_std_text_height_data = { 0.1, 10.0, 0.1 };
+static PropNumData prop_std_line_width_data = { 0.0f, 10.0f, 0.01f };
+static PropNumData prop_std_text_height_data = { 0.1f, 10.0f, 0.1f };
 static PropEnumData prop_std_text_align_data[] = {
   { N_("Left"), ALIGN_LEFT },
   { N_("Center"), ALIGN_CENTER },
@@ -481,10 +484,10 @@ extern PropEnumData prop_std_text_align_data[];
     N_("Line width"), NULL, &prop_std_line_width_data }
 #define PROP_STD_LINE_COLOUR \
   { "line_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD, \
-    N_("Line colour"), NULL, NULL }
+    N_("Line color"), NULL, NULL }
 #define PROP_STD_LINE_COLOUR_OPTIONAL \
   { "line_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL, \
-    N_("Line colour"), NULL, NULL }
+    N_("Line color"), NULL, NULL }
 #define PROP_STD_LINE_STYLE \
   { "line_style", PROP_TYPE_LINESTYLE, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD, \
     N_("Line style"), NULL, NULL }
@@ -494,13 +497,16 @@ extern PropEnumData prop_std_text_align_data[];
 
 #define PROP_STD_FILL_COLOUR \
   { "fill_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD, \
-    N_("Fill colour"), NULL, NULL }
+    N_("Fill color"), NULL, NULL }
 #define PROP_STD_FILL_COLOUR_OPTIONAL \
   { "fill_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL, \
-    N_("Fill colour"), NULL, NULL }
+    N_("Fill color"), NULL, NULL }
 #define PROP_STD_SHOW_BACKGROUND \
   { "show_background", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE, \
     N_("Draw background"), NULL, NULL }
+#define PROP_STD_SHOW_BACKGROUND_OPTIONAL \
+  { "show_background", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE|PROP_FLAG_OPTIONAL, \
+    N_("Draw background"), NULL, NULL }
 
 #define PROP_STD_START_ARROW \
   { "start_arrow", PROP_TYPE_ARROW, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD, \
@@ -509,27 +515,36 @@ extern PropEnumData prop_std_text_align_data[];
   { "end_arrow", PROP_TYPE_ARROW, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD, \
     N_("End arrow"), NULL, NULL }
 
-#define PROP_STD_TEXT \
-  { "text", PROP_TYPE_TEXT, PROP_FLAG_DONT_SAVE, \
+#define PROP_STD_TEXT_OPTIONS(options) \
+  { "text", PROP_TYPE_TEXT, (options), \
     N_("Text"), NULL, NULL }
-#define PROP_STD_SAVED_TEXT \
-  { "text", PROP_TYPE_TEXT, 0, \
-    N_("Text"), NULL, NULL }
-#define PROP_STD_TEXT_ALIGNMENT \
-  { "text_alignment", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE, \
+#define PROP_STD_TEXT PROP_STD_TEXT_OPTIONS(PROP_FLAG_DONT_SAVE)
+#define PROP_STD_SAVED_TEXT PROP_STD_TEXT_OPTIONS(0)
+
+#define PROP_STD_TEXT_ALIGNMENT_OPTIONS(options) \
+  { "text_alignment", PROP_TYPE_ENUM, (options), \
     N_("Text alignment"), NULL, prop_std_text_align_data }
+#define PROP_STD_TEXT_ALIGNMENT \
+        PROP_STD_TEXT_ALIGNMENT_OPTIONS(PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE)
+
+#define PROP_STD_TEXT_FONT_OPTIONS(options) \
+  { "text_font", PROP_TYPE_FONT, (options), N_("Font"), NULL, NULL }
 #define PROP_STD_TEXT_FONT \
-  { "text_font", PROP_TYPE_FONT, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE, \
-    N_("Font"), NULL, NULL }
-#define PROP_STD_TEXT_HEIGHT \
-  { "text_height", PROP_TYPE_REAL, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE, \
+        PROP_STD_TEXT_FONT_OPTIONS(PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE)
+
+#define PROP_STD_TEXT_HEIGHT_OPTIONS(options) \
+  { "text_height", PROP_TYPE_REAL, (options), \
     N_("Font size"), NULL, &prop_std_text_height_data }
+#define PROP_STD_TEXT_HEIGHT \
+        PROP_STD_TEXT_HEIGHT_OPTIONS(PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE)
+
+#define PROP_STD_TEXT_COLOUR_OPTIONS(options) \
+  { "text_colour", PROP_TYPE_COLOUR, (options), \
+    N_("Text color"), NULL, NULL }
 #define PROP_STD_TEXT_COLOUR \
-  { "text_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_STANDARD, \
-    N_("Text colour"), NULL, NULL }
+        PROP_STD_TEXT_COLOUR_OPTIONS(PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_STANDARD)
 #define PROP_STD_TEXT_COLOUR_OPTIONAL \
-  { "text_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL, \
-    N_("Text colour"), NULL, NULL }
+        PROP_STD_TEXT_COLOUR_OPTIONS(PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL)
 
 /* Convenience macros */
 #define PROP_NOTEBOOK_BEGIN(name) \
@@ -588,31 +603,6 @@ extern PropEnumData prop_std_text_align_data[];
 #define PROP_OFFSET_FRAME_END(name) \
   { "frame_" name "_end", PROP_TYPE_FRAME_END, 0}
 
-
+G_END_DECLS
 
 #endif
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
