@@ -30,6 +30,7 @@
 #include "properties.h"
 #include "propinternals.h"
 #include "dia_xml.h"
+#include "diaerror.h"
 
 /* ------------------------------------------------------------------------- */
 /* Construction of a list of properties from a filtered list of descriptors. */
@@ -56,7 +57,7 @@ gboolean pdtpp_do_load(const PropDescription *pdesc)
 { return (((pdesc->flags & PROP_FLAG_DONT_SAVE) == 0) ||
             ((pdesc->flags & PROP_FLAG_LOAD_ONLY) != 0)); } 
 gboolean pdtpp_defaults(const PropDescription *pdesc)
-{ return (pdesc->flags & (PROP_FLAG_DONT_MERGE|PROP_FLAG_NO_DEFAULTS)) == 0; } 
+{ return (pdesc->flags & PROP_FLAG_NO_DEFAULTS) == 0; } 
 gboolean pdtpp_do_not_save(const PropDescription *pdesc)
 { return (pdesc->flags & PROP_FLAG_DONT_SAVE) != 0; } 
 
@@ -141,26 +142,33 @@ prop_list_copy_empty(GPtrArray *plist)
   return dest;
 }
 
-void 
-prop_list_load(GPtrArray *props, DataNode data)
+gboolean 
+prop_list_load(GPtrArray *props, DataNode data, GError **err)
 {
   int i;
+  gboolean ret = TRUE;
 
   for (i = 0; i < props->len; i++) {
     Property *prop = g_ptr_array_index(props,i);
     AttributeNode attr = object_find_attribute(data, prop->name);
     DataNode data = attr ? attribute_first_data(attr) : NULL;
-    if (!attr && prop->descr->flags & PROP_FLAG_OPTIONAL) {
+    if ((!attr || !data) && prop->descr->flags & PROP_FLAG_OPTIONAL) {
       prop->experience |= PXP_NOTSET;
       continue;
     }
     if ((!attr) || (!data)) {
-      g_warning("No attribute %s (%p) or no data(%p) in this attribute",
-                prop->name,attr,data);
+      if (err && !*err)
+	*err = g_error_new (DIA_ERROR,
+                            DIA_ERROR_FORMAT,
+			    _("No attribute '%s' (%p) or no data(%p) in this attribute"),
+			    prop->name,attr,data);
+      prop->experience |= PXP_NOTSET;
+      ret = FALSE;
       continue;
     }
     prop->ops->load(prop,attr,data);
-  }    
+  }
+  return ret;
 }
 
 void 
