@@ -33,20 +33,25 @@
 
 #include "pixmaps/actor.xpm"
 
+#define NUM_CONNECTIONS 9
+
 typedef struct _Actor Actor;
 struct _Actor {
   Element element;
 
-  ConnectionPoint connections[8];
+  ConnectionPoint connections[NUM_CONNECTIONS];
 
   Text *text;
   TextAttributes attrs;
+
+  Color line_color;
+  Color fill_color;
 };
 
 #define ACTOR_WIDTH 2.2
 #define ACTOR_HEIGHT 4.6
-#define ACTOR_HEAD 0.6
-#define ACTOR_BODY 4.0
+#define ACTOR_HEAD(h) (h*0.6/4.6)
+#define ACTOR_BODY(h) (h*4.0/4.6)
 #define ACTOR_LINEWIDTH 0.1
 #define ACTOR_MARGIN_X 0.3
 #define ACTOR_MARGIN_Y 0.3
@@ -59,12 +64,12 @@ static ObjectChange* actor_move_handle(Actor *actor, Handle *handle,
 				       HandleMoveReason reason, ModifierKeys modifiers);
 static ObjectChange* actor_move(Actor *actor, Point *to);
 static void actor_draw(Actor *actor, DiaRenderer *renderer);
-static Object *actor_create(Point *startpoint,
+static DiaObject *actor_create(Point *startpoint,
 			   void *user_data,
 			   Handle **handle1,
 			   Handle **handle2);
 static void actor_destroy(Actor *actor);
-static Object *actor_load(ObjectNode obj_node, int version,
+static DiaObject *actor_load(ObjectNode obj_node, int version,
 			  const char *filename);
 
 static PropDescription *actor_describe_props(Actor *actor);
@@ -82,7 +87,7 @@ static ObjectTypeOps actor_type_ops =
   (ApplyDefaultsFunc) NULL
 };
 
-ObjectType actor_type =
+DiaObjectType actor_type =
 {
   "UML - Actor",   /* name */
   0,                      /* version */
@@ -111,7 +116,9 @@ static PropDescription actor_props[] = {
   ELEMENT_COMMON_PROPERTIES,
   PROP_STD_TEXT_FONT,
   PROP_STD_TEXT_HEIGHT,
-  PROP_STD_TEXT_COLOUR,
+  PROP_STD_TEXT_COLOUR_OPTIONAL,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  PROP_STD_FILL_COLOUR_OPTIONAL, 
   { "text", PROP_TYPE_TEXT, 0, N_("Text"), NULL, NULL }, 
   PROP_DESC_END
 };
@@ -131,6 +138,8 @@ static PropOffset actor_offsets[] = {
   {"text_font",PROP_TYPE_FONT,offsetof(Actor,attrs.font)},
   {"text_height",PROP_TYPE_REAL,offsetof(Actor,attrs.height)},
   {"text_colour",PROP_TYPE_COLOUR,offsetof(Actor,attrs.color)},
+  {"line_colour",PROP_TYPE_COLOUR,offsetof(Actor,line_color)},
+  {"fill_colour",PROP_TYPE_COLOUR,offsetof(Actor,fill_color)},
   { NULL, 0, 0 },
 };
 
@@ -154,7 +163,7 @@ actor_set_props(Actor *actor, GPtrArray *props)
 static real
 actor_distance_from(Actor *actor, Point *point)
 {
-  Object *obj = &actor->element.object;
+  DiaObject *obj = &actor->element.object;
   return distance_rectangle_point(&obj->bounding_box, point);
 }
 
@@ -172,13 +181,17 @@ actor_move_handle(Actor *actor, Handle *handle,
 		  Point *to, ConnectionPoint *cp,
 		  HandleMoveReason reason, ModifierKeys modifiers)
 {
+  ObjectChange* oc;
+  
   assert(actor!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   assert(handle->id < 8);
 
-  return NULL;
+  oc = element_move_handle (&(actor->element), handle->id, to, cp, reason, modifiers);
+  actor_update_data (actor);
+  return oc;
 }
 
 static ObjectChange*
@@ -203,6 +216,7 @@ actor_draw(Actor *actor, DiaRenderer *renderer)
   real x, y, w, h;
   real r, r1;  
   Point ch, cb, p1, p2;
+  real actor_height;
 
   assert(actor != NULL);
   assert(renderer != NULL);
@@ -213,12 +227,13 @@ actor_draw(Actor *actor, DiaRenderer *renderer)
   y = elem->corner.y;
   w = elem->width;
   h = elem->height;
+  actor_height = elem->height - actor->text->height;
   
   renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
   renderer_ops->set_linewidth(renderer, ACTOR_LINEWIDTH);
   renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
-  r = ACTOR_HEAD;
+  r = ACTOR_HEAD(actor_height);
   r1 = 2*r;
   ch.x = x + w*0.5;
   ch.y = y + r + ACTOR_MARGIN_Y;
@@ -229,11 +244,11 @@ actor_draw(Actor *actor, DiaRenderer *renderer)
   renderer_ops->fill_ellipse(renderer, 
 			     &ch,
 			     r, r,
-			     &color_white);
+			     &actor->fill_color);
   renderer_ops->draw_ellipse(renderer, 
 			     &ch,
 			     r, r,
-			     &color_black);  
+			     &actor->line_color);  
   
   /* Arms */
   p1.x = ch.x - r1;
@@ -241,25 +256,25 @@ actor_draw(Actor *actor, DiaRenderer *renderer)
   p1.y = p2.y = ch.y + r;
   renderer_ops->draw_line(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &actor->line_color);
 
   p1.x = ch.x;
   p1.y = ch.y + r*0.5;
   /* body & legs  */
   renderer_ops->draw_line(renderer, 
 			   &p1, &cb,
-			   &color_black);
+			   &actor->line_color);
 
   p2.x = ch.x - r1;
-  p2.y = y + ACTOR_BODY;
+  p2.y = y + ACTOR_BODY(actor_height);
   renderer_ops->draw_line(renderer, 
 			   &cb, &p2,
-			   &color_black);
+			   &actor->line_color);
   
   p2.x =  ch.x + r1;
   renderer_ops->draw_line(renderer, 
 			   &cb, &p2,
-			   &color_black);
+			   &actor->line_color);
   
   text_draw(actor->text, renderer);
 }
@@ -268,46 +283,28 @@ static void
 actor_update_data(Actor *actor)
 {
   Element *elem = &actor->element;
-  Object *obj = &elem->object;
+  DiaObject *obj = &elem->object;
   Rectangle text_box;
   Point p;
+  real actor_height;
   
   text_calc_boundingbox(actor->text, &text_box);
 
-  elem->width = ACTOR_WIDTH + ACTOR_MARGIN_X;
-  elem->height = ACTOR_HEIGHT+actor->text->height;
+  /* minimum size */
+  if (elem->width < ACTOR_WIDTH + ACTOR_MARGIN_X)
+    elem->width = ACTOR_WIDTH + ACTOR_MARGIN_X;
+  if (elem->height < ACTOR_HEIGHT + actor->text->height)
+    elem->height = ACTOR_HEIGHT + actor->text->height;
+  actor_height = elem->height - actor->text->height;
 
   /* Update connections: */
-  actor->connections[0].pos.x = elem->corner.x;
-  actor->connections[0].pos.y = elem->corner.y;
-  actor->connections[0].directions = DIR_NORTH|DIR_WEST;
-  actor->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
-  actor->connections[1].pos.y = elem->corner.y;
-  actor->connections[1].directions = DIR_NORTH;
-  actor->connections[2].pos.x = elem->corner.x + elem->width;
-  actor->connections[2].pos.y = elem->corner.y;
-  actor->connections[2].directions = DIR_NORTH|DIR_EAST;
-  actor->connections[3].pos.x = elem->corner.x;
-  actor->connections[3].pos.y = elem->corner.y + elem->height / 2.0;
-  actor->connections[3].directions = DIR_WEST;
-  actor->connections[4].pos.x = elem->corner.x + elem->width;
-  actor->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
-  actor->connections[4].directions = DIR_EAST;
-  actor->connections[5].pos.x = elem->corner.x;
-  actor->connections[5].pos.y = elem->corner.y + elem->height;
-  actor->connections[5].directions = DIR_SOUTH|DIR_WEST;
-  actor->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
-  actor->connections[6].pos.y = elem->corner.y + elem->height;
-  actor->connections[6].directions = DIR_SOUTH;
-  actor->connections[7].pos.x = elem->corner.x + elem->width;
-  actor->connections[7].pos.y = elem->corner.y + elem->height;
-  actor->connections[7].directions = DIR_SOUTH|DIR_EAST;
+  element_update_connections_rectangle(elem, actor->connections);
   
   element_update_boundingbox(elem);
 
   p = elem->corner;
   p.x += elem->width/2;
-  p.y +=  ACTOR_HEIGHT + actor->text->ascent;
+  p.y +=  actor_height + actor->text->ascent;
   text_set_position(actor->text, &p);
 
   /* Add bounding box for text: */
@@ -320,7 +317,7 @@ actor_update_data(Actor *actor)
   element_update_handles(elem);
 }
 
-static Object *
+static DiaObject *
 actor_create(Point *startpoint,
 	       void *user_data,
   	       Handle **handle1,
@@ -328,7 +325,7 @@ actor_create(Point *startpoint,
 {
   Actor *actor;
   Element *elem;
-  Object *obj;
+  DiaObject *obj;
   Point p;
   DiaFont *font;
   int i;
@@ -343,6 +340,9 @@ actor_create(Point *startpoint,
   elem->width = ACTOR_WIDTH;
   elem->height = ACTOR_HEIGHT;
 
+  actor->line_color = attributes_get_foreground();
+  actor->fill_color = attributes_get_background();
+
   font = dia_font_new_from_style (DIA_FONT_SANS, 0.8);
   p = *startpoint;
   p.x += ACTOR_MARGIN_X;
@@ -354,13 +354,14 @@ actor_create(Point *startpoint,
   
   text_get_attributes(actor->text,&actor->attrs);
   
-  element_init(elem, 8, 8);
+  element_init(elem, 8, NUM_CONNECTIONS);
   
-  for (i=0;i<8;i++) {
+  for (i=0;i<NUM_CONNECTIONS;i++) {
     obj->connections[i] = &actor->connections[i];
     actor->connections[i].object = obj;
     actor->connections[i].connected = NULL;
   }
+  actor->connections[8].flags = CP_FLAGS_MAIN;
   elem->extra_spacing.border_trans = ACTOR_LINEWIDTH/2.0;
   actor_update_data(actor);
 
@@ -381,7 +382,7 @@ actor_destroy(Actor *actor)
   element_destroy(&actor->element);
 }
 
-static Object *
+static DiaObject *
 actor_load(ObjectNode obj_node, int version, const char *filename)
 {
   return object_load_using_properties(&actor_type,
