@@ -40,10 +40,12 @@
 
 typedef struct _Objet Objet;
 
+#define NUM_CONNECTIONS 9
+
 struct _Objet {
   Element element;
 
-  ConnectionPoint connections[8];
+  ConnectionPoint connections[NUM_CONNECTIONS];
   
   char *stereotype;
   Text *text;
@@ -80,12 +82,12 @@ static ObjectChange* objet_move_handle(Objet *ob, Handle *handle,
 				       HandleMoveReason reason, ModifierKeys modifiers);
 static ObjectChange* objet_move(Objet *ob, Point *to);
 static void objet_draw(Objet *ob, DiaRenderer *renderer);
-static Object *objet_create(Point *startpoint,
+static DiaObject *objet_create(Point *startpoint,
 			    void *user_data,
 			    Handle **handle1,
 			    Handle **handle2);
 static void objet_destroy(Objet *ob);
-static Object *objet_load(ObjectNode obj_node, int version,
+static DiaObject *objet_load(ObjectNode obj_node, int version,
 			  const char *filename);
 static PropDescription *objet_describe_props(Objet *objet);
 static void objet_get_props(Objet *objet, GPtrArray *props);
@@ -102,7 +104,7 @@ static ObjectTypeOps objet_type_ops =
 };
 
 /* Non-nice typo, needed for backwards compatibility. */
-ObjectType objet_type =
+DiaObjectType objet_type =
 {
   "UML - Objet",   /* name */
   0,                      /* version */
@@ -111,7 +113,7 @@ ObjectType objet_type =
   &objet_type_ops       /* ops */
 };
 
-ObjectType umlobject_type =
+DiaObjectType umlobject_type =
 {
   "UML - Object",   /* name */
   0,                      /* version */
@@ -139,7 +141,8 @@ static ObjectOps objet_ops = {
 
 static PropDescription objet_props[] = {
   ELEMENT_COMMON_PROPERTIES,
-  PROP_STD_TEXT_COLOUR_OPTIONAL,
+  /* can't use PROP_STD_TEXT_COLOUR_OPTIONAL cause it has PROP_FLAG_DONT_SAVE. It is designed to fill the Text object - not some subset */
+  PROP_STD_TEXT_COLOUR_OPTIONS(PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL),
   PROP_STD_LINE_COLOUR_OPTIONAL, 
   PROP_STD_FILL_COLOUR_OPTIONAL, 
   { "text", PROP_TYPE_TEXT, 0, N_("Text"), NULL, NULL },
@@ -209,7 +212,7 @@ objet_set_props(Objet *objet, GPtrArray *props)
 static real
 objet_distance_from(Objet *ob, Point *point)
 {
-  Object *obj = &ob->element.object;
+  DiaObject *obj = &ob->element.object;
   return distance_rectangle_point(&obj->bounding_box, point);
 }
 
@@ -300,6 +303,8 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
   
   text_draw(ob->text, renderer);
 
+  renderer_ops->set_font(renderer, ob->text->font, ob->text->height);
+
   if ((ob->st_stereotype != NULL) && (ob->st_stereotype[0] != '\0')) {
       renderer_ops->draw_string(renderer,
 				 ob->st_stereotype,
@@ -315,16 +320,16 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
   }
 
   /* Is there a better way to underline? */
-  p1.x = x + (w - ob->text->max_width)/2;
-  p1.y = ob->text->position.y + ob->text->descent;
-  p2.x = p1.x + ob->text->max_width;
+  p1.x = x + (w - text_get_max_width(ob->text))/2;
+  p1.y = ob->text->position.y + text_get_descent(ob->text);
+  p2.x = p1.x + text_get_max_width(ob->text);
   p2.y = p1.y;
   
   renderer_ops->set_linewidth(renderer, OBJET_LINEWIDTH);
     
   for (i=0; i<ob->text->numlines; i++) { 
-    p1.x = x + (w - ob->text->row_width[i])/2;
-    p2.x = p1.x + ob->text->row_width[i];
+    p1.x = x + (w - text_get_line_width(ob->text, i))/2;
+    p2.x = p1.x + text_get_line_width(ob->text, i);
     renderer_ops->draw_line(renderer,
 			     &p1, &p2,
 			     &ob->text_color);
@@ -348,7 +353,7 @@ static void
 objet_update_data(Objet *ob)
 {
   Element *elem = &ob->element;
-  Object *obj = &elem->object;
+  DiaObject *obj = &elem->object;
   DiaFont *font;
   Point p1, p2;
   real h, w = 0;
@@ -413,37 +418,14 @@ objet_update_data(Objet *ob)
   elem->height = h - elem->corner.y;
 
   /* Update connections: */
-  ob->connections[0].pos = elem->corner;
-  ob->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
-  ob->connections[1].pos.y = elem->corner.y;
-  ob->connections[2].pos.x = elem->corner.x + elem->width;
-  ob->connections[2].pos.y = elem->corner.y;
-  ob->connections[3].pos.x = elem->corner.x;
-  ob->connections[3].pos.y = elem->corner.y + elem->height / 2.0;
-  ob->connections[4].pos.x = elem->corner.x + elem->width;
-  ob->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
-  ob->connections[5].pos.x = elem->corner.x;
-  ob->connections[5].pos.y = elem->corner.y + elem->height;
-  ob->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
-  ob->connections[6].pos.y = elem->corner.y + elem->height;
-  ob->connections[7].pos.x = elem->corner.x + elem->width;
-  ob->connections[7].pos.y = elem->corner.y + elem->height;
+  element_update_connections_rectangle(elem, ob->connections);
   
-  ob->connections[0].directions = DIR_NORTH|DIR_WEST;
-  ob->connections[1].directions = DIR_NORTH;
-  ob->connections[2].directions = DIR_NORTH|DIR_EAST;
-  ob->connections[3].directions = DIR_WEST;
-  ob->connections[4].directions = DIR_EAST;
-  ob->connections[5].directions = DIR_SOUTH|DIR_WEST;
-  ob->connections[6].directions = DIR_SOUTH;
-  ob->connections[7].directions = DIR_SOUTH|DIR_EAST;
-                                                                                                
-                                                                                                  element_update_boundingbox(elem);
+  element_update_boundingbox(elem);
   obj->position = elem->corner;
   element_update_handles(elem);
 }
 
-static Object *
+static DiaObject *
 objet_create(Point *startpoint,
 		    void *user_data,
 		    Handle **handle1,
@@ -451,7 +433,7 @@ objet_create(Point *startpoint,
 {
   Objet *ob;
   Element *elem;
-  Object *obj;
+  DiaObject *obj;
   Point p;
   DiaFont *font;
   int i;
@@ -489,13 +471,14 @@ objet_create(Point *startpoint,
 
   dia_font_unref(font);
   
-  element_init(elem, 8, 8);
+  element_init(elem, 8, NUM_CONNECTIONS);
   
-  for (i=0;i<8;i++) {
+  for (i=0;i<NUM_CONNECTIONS;i++) {
     obj->connections[i] = &ob->connections[i];
     ob->connections[i].object = obj;
     ob->connections[i].connected = NULL;
   }
+  ob->connections[8].flags = CP_FLAGS_MAIN;
   elem->extra_spacing.border_trans = OBJET_BORDERWIDTH/2.0;
   objet_update_data(ob);
 
@@ -523,7 +506,7 @@ objet_destroy(Objet *ob)
   element_destroy(&ob->element);
 }
 
-static Object *
+static DiaObject *
 objet_load(ObjectNode obj_node, int version, const char *filename)
 {
   return object_load_using_properties(&objet_type,
