@@ -31,6 +31,8 @@
 #include <glib.h>
 #include <errno.h>
 
+#include <glib/gstdio.h>
+
 #include "intl.h"
 #include "message.h"
 #include "geometry.h"
@@ -84,12 +86,6 @@ write_uint32(FILE *fp, guint32 n)
     putc(n & 0xff, fp); 
 }
 
-static void
-write_int32(FILE *fp, gint32 n)
-{
-    write_uint32(fp, (guint32) n);
-}
-
 static void
 write_colour(FILE *fp, Color *c)
 {
@@ -1151,10 +1147,11 @@ export_cgm(DiagramData *data, const gchar *filename,
     Rectangle *extent;
     gint len;
 
-    file = fopen(filename, "wb");
+    file = g_fopen(filename, "wb");
 
     if (file == NULL) {
-	message_error(_("Can't open output file %s: %s\n"), filename, strerror(errno));
+	message_error(_("Can't open output file %s: %s\n"), 
+		      dia_message_filename(filename), strerror(errno));
 	return;
     }
 
@@ -1310,8 +1307,6 @@ cgm_renderer_get_type (void)
 static void
 cgm_renderer_finalize (GObject *object)
 {
-  CgmRenderer *cgm_renderer = CGM_RENDERER (object);
-
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -1369,6 +1364,17 @@ static DiaExportFilter cgm_export_filter = {
 
 
 /* --- dia plug-in interface --- */
+static gboolean
+_plugin_can_unload (PluginInfo *info)
+{
+  return TRUE;
+}
+
+static void
+_plugin_unload (PluginInfo *info)
+{
+  filter_unregister_export(&cgm_export_filter);
+}
 
 DIA_PLUGIN_CHECK_INIT
 
@@ -1377,7 +1383,8 @@ dia_plugin_init(PluginInfo *info)
 {
     if (!dia_plugin_info_init(info, "CGM",
 			      _("Computer Graphics Metafile export filter"),
-			      NULL, NULL))
+			      _plugin_can_unload,
+                              _plugin_unload))
 	return DIA_PLUGIN_INIT_ERROR;
 
     filter_register_export(&cgm_export_filter);
