@@ -1,10 +1,18 @@
 import sys, os
 import dia
 
+# Please don't include pygtk or gtk here. There
+# are quite some PyDia plug-ins which don't require
+# i.e. dont have any (Py)Gtk code
+
 # find system python plugin dir
 curdir = os.path.dirname(__file__)
 plugindir = os.path.join(curdir, 'python')
 
+#If we're running from source there is no plugin dir
+if not os.path.exists(plugindir):
+    plugindir = curdir
+
 if os.path.isdir(plugindir):
     # import all plugins found in system plugin dir
     sys.path.insert(0, plugindir)
