@@ -11,11 +11,13 @@
 
 #include <string.h>
 #include <math.h>
-#include <glib.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <locale.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include "intl.h"
 #include "message.h"
 #include "geometry.h"
@@ -36,6 +38,9 @@
 #define XFIG_RENDERER_CLASS(klass)   (G_TYPE_CHECK_CLASS_CAST ((klass), XFIG_TYPE_RENDERER, XfigRendererClass))
 #define XFIG_IS_RENDERER(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), XFIG_TYPE_RENDERER))
 #define XFIG_RENDERER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), XFIG_TYPE_RENDERER, XfigRendererClass))
+#define DTOSTR_BUF_SIZE G_ASCII_DTOSTR_BUF_SIZE
+#define xfig_dtostr(buf,d) \
+	g_ascii_formatd(buf, sizeof(buf), "%f", d)
 
 GType xfig_renderer_get_type (void) G_GNUC_CONST;
 
@@ -71,6 +76,12 @@ struct _XfigRenderer
   gchar *warnings[MAX_WARNING];
 };
 
+/* check whether there exists an arrow head */
+static int hasArrow(Arrow *arrow)
+{
+  return (!arrow || ARROW_NONE==arrow->type) ? 0 : 1;
+}
+
 static void begin_render(DiaRenderer *self);
 static void end_render(DiaRenderer *renderer);
 static void set_linewidth(DiaRenderer *self, real linewidth);
@@ -86,6 +97,18 @@ static void draw_line(DiaRenderer *self,
 static void draw_polyline(DiaRenderer *self, 
 			  Point *points, int num_points, 
 			  Color *color);
+static void draw_line_with_arrows(DiaRenderer *self, 
+				  Point *start, Point *end, 
+				  real line_width,
+				  Color *color,
+				  Arrow *start_arrow,
+				  Arrow *end_arrow);
+static void draw_polyline_with_arrows(DiaRenderer *self, 
+				      Point *points, int num_points, 
+				      real line_width,
+				      Color *color,
+				      Arrow *start_arrow,
+				      Arrow *end_arrow);
 static void draw_polygon(DiaRenderer *self, 
 			 Point *points, int num_points, 
 			 Color *color);
@@ -103,6 +126,14 @@ static void draw_arc(DiaRenderer *self,
 		     real width, real height,
 		     real angle1, real angle2,
 		     Color *colour);
+static void draw_arc_with_arrows(DiaRenderer *self, 
+				 Point *startpoint,
+				 Point *endpoint,
+				 Point *midpoint,
+				 real line_width,
+				 Color *colour,
+				 Arrow *start_arrow,
+				 Arrow *end_arrow);
 static void fill_arc(DiaRenderer *self, 
 		     Point *center,
 		     real width, real height,
@@ -120,6 +151,13 @@ static void draw_bezier(DiaRenderer *self,
 			BezPoint *points,
 			int numpoints,
 			Color *colour);
+static void draw_bezier_with_arrows(DiaRenderer *self, 
+				    BezPoint *points,
+				    int numpoints,
+				    real line_width,
+				    Color *colour,
+				    Arrow *start_arrow,
+				    Arrow *end_arrow);
 static void fill_bezier(DiaRenderer *self, 
 			BezPoint *points, /* Last point must be same as first point */
 			int numpoints,
@@ -133,7 +171,7 @@ static void draw_image(DiaRenderer *self,
 		       real width, real height,
 		       DiaImage image);
 static void draw_object(DiaRenderer *self,
-			Object *object);
+			DiaObject *object);
 
 static void xfig_renderer_class_init (XfigRendererClass *klass);
 
@@ -170,8 +208,6 @@ xfig_renderer_get_type (void)
 static void
 xfig_renderer_finalize (GObject *object)
 {
-  XfigRenderer *xfig_renderer = XFIG_RENDERER (object);
-
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -218,6 +254,10 @@ xfig_renderer_class_init (XfigRendererClass *klass)
 
   renderer_class->draw_image = draw_image;
 
+  renderer_class->draw_line_with_arrows = draw_line_with_arrows;
+  renderer_class->draw_polyline_with_arrows = draw_polyline_with_arrows;
+  renderer_class->draw_arc_with_arrows = draw_arc_with_arrows;
+  renderer_class->draw_bezier_with_arrows = draw_bezier_with_arrows;
   renderer_class->draw_object = draw_object;
 }
 
@@ -253,7 +293,11 @@ figLineStyle(XfigRenderer *renderer)
 static int 
 figLineWidth(XfigRenderer *renderer) 
 {
-  return (int)((renderer->linewidth / 2.54) * 80.0);
+  int width = 0;
+  /* Minimal line width in fig diagrams. */
+  if (renderer->linewidth <= 0.3175) width = 1;
+  else width = (int)((renderer->linewidth / 2.54) * 80.0);
+  return width;
 }
 
 /* Must be called before outputting anything that uses this color,
@@ -298,6 +342,18 @@ figColor(XfigRenderer *renderer, Color *color)
   return 0;
 }
 
+static real 
+figCoord(XfigRenderer *renderer, real coord) 
+{
+  return (coord/2.54)*1200.0;
+}
+
+static real
+figAltCoord(XfigRenderer *renderer, real coord)
+{
+  return (coord/2.54)*80.0;
+}
+
 static int 
 figDepth(XfigRenderer *renderer) 
 {
@@ -307,7 +363,7 @@ figDepth(XfigRenderer *renderer)
 static real 
 figDashLength(XfigRenderer *renderer) 
 {
-  return (renderer->dashlength / 2.54) * 80.0;
+  return figAltCoord(renderer, renderer->dashlength);
 }
 
 static int 
@@ -322,18 +378,6 @@ figCapsStyle(XfigRenderer *renderer)
   return renderer->capsmode;
 }
 
-static int 
-figCoord(XfigRenderer *renderer, real coord) 
-{
-  return (int)((coord/2.54)*1200.0);
-}
-
-static real 
-figFloatCoord(XfigRenderer *renderer, real coord) 
-{
-  return (coord/2.54)*1200.0;
-}
-
 static int 
 figAlignment(XfigRenderer *renderer, int alignment) 
 {
@@ -380,6 +424,9 @@ figText(XfigRenderer *renderer, const guchar *text)
     if (text[i] > 127) {
       sprintf(&returntext[j], "\\%03o", text[i]);
       j+=3;
+    } else if ('\\'==text[i]) { /* backslash must be quoted in xfig */
+      returntext[j++] = '\\';
+      returntext[j]   = '\\';
     } else {
       returntext[j] = text[i];
     }
@@ -388,6 +435,43 @@ figText(XfigRenderer *renderer, const guchar *text)
   return returntext;
 }
 
+static void
+figArrow(XfigRenderer *renderer, Arrow *arrow, real line_width)
+{
+  int type, style;
+  gchar lw_buf[DTOSTR_BUF_SIZE];
+  gchar aw_buf[DTOSTR_BUF_SIZE];
+  gchar al_buf[DTOSTR_BUF_SIZE];
+  
+  switch (arrow->type) {
+  case ARROW_NONE:
+    return;
+  case ARROW_LINES:             /* {open arrow} */
+    type = 0; style = 0; break;
+  case ARROW_UNFILLED_TRIANGLE:
+  case ARROW_HOLLOW_TRIANGLE:   /* {blanked arrow} */
+    type = 1; style = 0; break;
+  case ARROW_FILLED_TRIANGLE:   /* {filled arrow} */
+    type = 1; style = 1; break;
+  case ARROW_HOLLOW_DIAMOND:   
+    type = 3; style = 0; break;
+  case ARROW_FILLED_DIAMOND:
+    type = 3; style = 1; break;
+  default:
+    message_warning(_("FIG format has no equivalent of arrow style %s, using simple arrow.\n"), arrow_types[arrow->type].name);
+    /* Notice fallthrough */
+  case ARROW_FILLED_CONCAVE:
+    type = 2; style = 1; break;
+  case ARROW_BLANKED_CONCAVE:
+    type = 2; style = 0; break;
+  }
+  fprintf(renderer->file, "  %d %d %s %s %s\n",
+	  type, style,
+	  xfig_dtostr(lw_buf, figAltCoord(renderer, line_width)), 
+	  xfig_dtostr(aw_buf, figCoord(renderer, arrow->width)),
+	  xfig_dtostr(al_buf, figCoord(renderer, arrow->length)) );	  
+}
+
 static void 
 begin_render(DiaRenderer *self) 
 {
@@ -480,20 +564,56 @@ draw_line(DiaRenderer *self,
           Color *color) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %f %d %d 0 0 0 2\n",
+  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %s %d %d 0 0 0 2\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer), 
-	  figCapsStyle(renderer));
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer), figCapsStyle(renderer) );
   fprintf(renderer->file, "\t%d %d %d %d\n",
-	  figCoord(renderer, start->x), figCoord(renderer, start->y), 
-	  figCoord(renderer, end->x), figCoord(renderer, end->y));
+	  (int)figCoord(renderer, start->x), (int)figCoord(renderer, start->y), 
+	  (int)figCoord(renderer, end->x), (int)figCoord(renderer, end->y));
+}
+
+static void 
+draw_line_with_arrows(DiaRenderer *self, 
+		      Point *start, Point *end, 
+		      real line_width,
+		      Color *color,
+		      Arrow *start_arrow,
+		      Arrow *end_arrow) 
+{
+  XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
+
+  if (renderer->color_pass) {
+    figCheckColor(renderer, color);
+    return;
+  }
+
+  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %s %d %d 0 %d %d 2\n",
+	  figLineStyle(renderer), figLineWidth(renderer), 
+	  figColor(renderer, color), figDepth(renderer),
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer), 
+	  figCapsStyle(renderer),
+	  hasArrow(end_arrow),
+	  hasArrow(start_arrow));
+  if (hasArrow(end_arrow)) {
+    figArrow(renderer, end_arrow, line_width);
+  }
+  if (hasArrow(start_arrow)) {
+    figArrow(renderer, start_arrow, line_width);
+  }
+  fprintf(renderer->file, "\t%d %d %d %d\n",
+	  (int)figCoord(renderer, start->x), (int)figCoord(renderer, start->y), 
+	  (int)figCoord(renderer, end->x), (int)figCoord(renderer, end->y));
 }
 
 static void 
@@ -503,21 +623,62 @@ draw_polyline(DiaRenderer *self,
 {
   int i;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %f %d %d 0 0 0 %d\n",
+  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %s %d %d 0 0 0 %d\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer),
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer),
 	  figCapsStyle(renderer), num_points);
   fprintf(renderer->file, "\t");
   for (i = 0; i < num_points; i++) {
     fprintf(renderer->file, "%d %d ",
-	    figCoord(renderer, points[i].x), figCoord(renderer, points[i].y));
+	    (int)figCoord(renderer, points[i].x), (int)figCoord(renderer, points[i].y));
+  }
+  fprintf(renderer->file, "\n");
+}
+
+static void 
+draw_polyline_with_arrows(DiaRenderer *self, 
+			  Point *points, int num_points, 
+			  real line_width,
+			  Color *color,
+			  Arrow *start_arrow,
+			  Arrow *end_arrow) 
+{
+  int i;
+  XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
+
+  if (renderer->color_pass) {
+    figCheckColor(renderer, color);
+    return;
+  }
+
+  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %s %d %d 0 %d %d %d\n",
+	  figLineStyle(renderer), figLineWidth(renderer), 
+	  figColor(renderer, color), figDepth(renderer),
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer),
+	  figCapsStyle(renderer),
+	  hasArrow(end_arrow),
+	  hasArrow(start_arrow), num_points);
+  if (hasArrow(end_arrow)) {
+    figArrow(renderer, end_arrow, line_width);
+  }
+  if (hasArrow(start_arrow)) {
+    figArrow(renderer, start_arrow, line_width);
+  }
+  fprintf(renderer->file, "\t");
+  for (i = 0; i < num_points; i++) {
+    fprintf(renderer->file, "%d %d ",
+	    (int)figCoord(renderer, points[i].x), (int)figCoord(renderer, points[i].y));
   }
   fprintf(renderer->file, "\n");
 }
@@ -529,24 +690,26 @@ draw_polygon(DiaRenderer *self,
 {
   int i;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 3 %d %d %d 0 %d 0 -1 %f %d %d 0 0 0 %d\n",
+  fprintf(renderer->file, "2 3 %d %d %d 0 %d 0 -1 %s %d %d 0 0 0 %d\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer),
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer),
 	  figCapsStyle(renderer), num_points+1);
   fprintf(renderer->file, "\t");
   for (i = 0; i < num_points; i++) {
     fprintf(renderer->file, "%d %d ",
-	    figCoord(renderer, points[i].x), figCoord(renderer, points[i].y));
+	    (int)figCoord(renderer, points[i].x), (int)figCoord(renderer, points[i].y));
   }
   fprintf(renderer->file, "%d %d\n",
-	  figCoord(renderer, points[0].x), figCoord(renderer, points[0].y));
+	  (int)figCoord(renderer, points[0].x), (int)figCoord(renderer, points[0].y));
 }
 
 static void 
@@ -556,25 +719,27 @@ fill_polygon(DiaRenderer *self,
 {
   int i;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 3 %d 0 %d %d %d 0 20 %f %d %d 0 0 0 %d\n",
+  fprintf(renderer->file, "2 3 %d 0 %d %d %d 0 20 %s %d %d 0 0 0 %d\n",
 	  figLineStyle(renderer), 
 	  figColor(renderer, color), figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer),
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer),
 	  figCapsStyle(renderer), num_points+1);
   fprintf(renderer->file, "\t");
   for (i = 0; i < num_points; i++) {
     fprintf(renderer->file, "%d %d ",
-	    figCoord(renderer, points[i].x), figCoord(renderer, points[i].y));
+	    (int)figCoord(renderer, points[i].x), (int)figCoord(renderer, points[i].y));
   }
   fprintf(renderer->file, "%d %d\n",
-	  figCoord(renderer, points[0].x), figCoord(renderer, points[0].y));
+	  (int)figCoord(renderer, points[0].x), (int)figCoord(renderer, points[0].y));
 }
 
 static void 
@@ -583,23 +748,25 @@ draw_rect(DiaRenderer *self,
           Color *color) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 3 %d %d %d 0 %d 0 -1 %f %d %d 0 0 0 5\n",
+  fprintf(renderer->file, "2 3 %d %d %d 0 %d 0 -1 %s %d %d 0 0 0 5\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer), 
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer), 
 	  figCapsStyle(renderer));
   fprintf(renderer->file, "\t%d %d %d %d %d %d %d %d %d %d\n",
-	  figCoord(renderer, ul_corner->x), figCoord(renderer, ul_corner->y), 
-	  figCoord(renderer, lr_corner->x), figCoord(renderer, ul_corner->y), 
-	  figCoord(renderer, lr_corner->x), figCoord(renderer, lr_corner->y), 
-	  figCoord(renderer, ul_corner->x), figCoord(renderer, lr_corner->y), 
-	  figCoord(renderer, ul_corner->x), figCoord(renderer, ul_corner->y));
+	  (int)figCoord(renderer, ul_corner->x), (int)figCoord(renderer, ul_corner->y), 
+	  (int)figCoord(renderer, lr_corner->x), (int)figCoord(renderer, ul_corner->y), 
+	  (int)figCoord(renderer, lr_corner->x), (int)figCoord(renderer, lr_corner->y), 
+	  (int)figCoord(renderer, ul_corner->x), (int)figCoord(renderer, lr_corner->y), 
+	  (int)figCoord(renderer, ul_corner->x), (int)figCoord(renderer, ul_corner->y));
 }
 
 static void 
@@ -608,24 +775,26 @@ fill_rect(DiaRenderer *self,
           Color *color) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 3 %d %d %d %d %d -1 20 %f %d %d 0 0 0 5\n",
+  fprintf(renderer->file, "2 3 %d %d %d %d %d -1 20 %s %d %d 0 0 0 5\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer), 
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer), 
 	  figCapsStyle(renderer));
   fprintf(renderer->file, "\t%d %d %d %d %d %d %d %d %d %d\n",
-	  figCoord(renderer, ul_corner->x), figCoord(renderer, ul_corner->y), 
-	  figCoord(renderer, lr_corner->x), figCoord(renderer, ul_corner->y), 
-	  figCoord(renderer, lr_corner->x), figCoord(renderer, lr_corner->y), 
-	  figCoord(renderer, ul_corner->x), figCoord(renderer, lr_corner->y), 
-	  figCoord(renderer, ul_corner->x), figCoord(renderer, ul_corner->y));
+	  (int)figCoord(renderer, ul_corner->x), (int)figCoord(renderer, ul_corner->y), 
+	  (int)figCoord(renderer, lr_corner->x), (int)figCoord(renderer, ul_corner->y), 
+	  (int)figCoord(renderer, lr_corner->x), (int)figCoord(renderer, lr_corner->y), 
+	  (int)figCoord(renderer, ul_corner->x), (int)figCoord(renderer, lr_corner->y), 
+	  (int)figCoord(renderer, ul_corner->x), (int)figCoord(renderer, ul_corner->y));
 }
 
 static void 
@@ -637,6 +806,9 @@ draw_arc(DiaRenderer *self,
 {
   Point first, second, last;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar dl_buf[DTOSTR_BUF_SIZE];
+  gchar cx_buf[DTOSTR_BUF_SIZE];
+  gchar cy_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
@@ -655,20 +827,88 @@ draw_arc(DiaRenderer *self,
   last.x += (width/2.0)*cos(angle2);
   last.y -= (height/2.0)*sin(angle2);
 
-  fprintf(renderer->file, "5 1 %d %d %d %d %d 0 -1 %f %d 1 0 0 %f %f %d %d %d %d %d %d\n",
+  fprintf(renderer->file, "5 1 %d %d %d %d %d 0 -1 %s %d 1 0 0 %s %s %d %d %d %d %d %d\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer),
+	  xfig_dtostr(dl_buf, figDashLength(renderer)),
 	  figCapsStyle(renderer),
-	  figFloatCoord(renderer, center->x), 
-	  figFloatCoord(renderer, center->y),
-	  figCoord(renderer, first.x), figCoord(renderer, first.y), 
-	  figCoord(renderer, second.x), figCoord(renderer, second.y), 
-	  figCoord(renderer, last.x), figCoord(renderer, last.y));
+	  xfig_dtostr(cx_buf, figCoord(renderer, center->x)), 
+	  xfig_dtostr(cy_buf, figCoord(renderer, center->y)),
+	  (int)figCoord(renderer, first.x), (int)figCoord(renderer, first.y), 
+	  (int)figCoord(renderer, second.x), (int)figCoord(renderer, second.y), 
+	  (int)figCoord(renderer, last.x), (int)figCoord(renderer, last.y));
 
 }
 
+static void 
+draw_arc_with_arrows(DiaRenderer *self, 
+		     Point *startpoint,
+		     Point *endpoint,
+		     Point *midpoint,
+		     real line_width,
+		     Color *color,
+		     Arrow *start_arrow,
+		     Arrow *end_arrow) 
+{
+  Point center;
+  Point t1, t2;
+  Point l1, l2;
+  Point p1, p2;
+  XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar dl_buf[DTOSTR_BUF_SIZE];
+  gchar cx_buf[DTOSTR_BUF_SIZE];
+  gchar cy_buf[DTOSTR_BUF_SIZE];
+
+  if (renderer->color_pass) {
+    figCheckColor(renderer, color);
+    return;
+  }
+
+  /* Since the lines between points on the circle form tangents, the
+   * intersection between the perpendiculars to those give the center.
+   */
+  
+  t1 = t2 = *midpoint;
+  point_sub(&t1, startpoint);
+  point_sub(&t2, endpoint);
+  point_scale(&t1, .5);
+  point_scale(&t2, .5);
+  p1 = p2 = *midpoint;
+  point_add(&p1, &t1);
+  point_add(&p2, &t2);
+  /* Now p1 and p2 are the midpoints of the tangents */
+  point_get_perp(&l1, &t1);
+  point_get_perp(&l2, &t2);
+  /* Now p1->t1 and p2->t2 give vectors crossing at the center.
+   * The direction of the vector (in or out) doesn't matter. */
+  
+
+  center.x = 0.0;
+  center.y = 0.0;
+
+  fprintf(renderer->file, "5 1 %d %d %d %d %d 0 -1 %s %d 1 %d %d %s %s %d %d %d %d %d %d\n",
+	  figLineStyle(renderer), figLineWidth(renderer), 
+	  figColor(renderer, color), figColor(renderer, color),
+	  figDepth(renderer),
+	  xfig_dtostr(dl_buf, figDashLength(renderer)),
+	  figCapsStyle(renderer),
+	  hasArrow(end_arrow),
+	  hasArrow(start_arrow),
+	  xfig_dtostr(cx_buf, figCoord(renderer, center.x)),
+	  xfig_dtostr(cy_buf, figCoord(renderer, center.y)),
+	  (int)figCoord(renderer, startpoint->x), (int)figCoord(renderer, startpoint->y), 
+	  (int)figCoord(renderer, midpoint->x), (int)figCoord(renderer, midpoint->y), 
+	  (int)figCoord(renderer, endpoint->x), (int)figCoord(renderer, endpoint->y));
+
+  if (hasArrow(end_arrow)) {
+    figArrow(renderer, end_arrow, line_width);
+  }
+  if (hasArrow(start_arrow)) {
+    figArrow(renderer, start_arrow, line_width);
+  }
+}
+
 static void 
 fill_arc(DiaRenderer *self, 
          Point *center,
@@ -678,6 +918,9 @@ fill_arc(DiaRenderer *self,
 {
   Point first, second, last;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar dl_buf[DTOSTR_BUF_SIZE];
+  gchar cx_buf[DTOSTR_BUF_SIZE];
+  gchar cy_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
@@ -696,17 +939,17 @@ fill_arc(DiaRenderer *self,
   last.x += (width/2.0)*cos(angle2);
   last.y -= (height/2.0)*sin(angle2);
 
-  fprintf(renderer->file, "5 1 %d %d %d %d %d 20 0 %f %d 1 0 0 %f %f %d %d %d %d %d %d\n",
+  fprintf(renderer->file, "5 1 %d %d %d %d %d 20 0 %s %d 1 0 0 %s %s %d %d %d %d %d %d\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer),
+	  xfig_dtostr(dl_buf, figDashLength(renderer)),
 	  figCapsStyle(renderer),
-	  figFloatCoord(renderer, center->x), 
-	  figFloatCoord(renderer, center->y),
-	  figCoord(renderer, first.x), figCoord(renderer, first.y), 
-	  figCoord(renderer, second.x), figCoord(renderer, second.y), 
-	  figCoord(renderer, last.x), figCoord(renderer, last.y));
+	  xfig_dtostr(cx_buf, figCoord(renderer, center->x)), 
+	  xfig_dtostr(cy_buf, figCoord(renderer, center->y)),
+	  (int)figCoord(renderer, first.x), (int)figCoord(renderer, first.y), 
+	  (int)figCoord(renderer, second.x), (int)figCoord(renderer, second.y), 
+	  (int)figCoord(renderer, last.x), (int)figCoord(renderer, last.y));
 
 }
 
@@ -717,20 +960,21 @@ draw_ellipse(DiaRenderer *self,
              Color *color) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "1 1 %d %d %d -1 %d 0 -1 %f 1 0.0 %d %d %d %d 0 0 0 0\n",
+  fprintf(renderer->file, "1 1 %d %d %d -1 %d 0 -1 %s 1 0.0 %d %d %d %d 0 0 0 0\n",
 	  figLineStyle(renderer), 
 	  figLineWidth(renderer), 
 	  figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer),
-	  figCoord(renderer, center->x), figCoord(renderer, center->y),
-	  figCoord(renderer, width/2), figCoord(renderer, height/2));
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  (int)figCoord(renderer, center->x), (int)figCoord(renderer, center->y),
+	  (int)figCoord(renderer, width/2), (int)figCoord(renderer, height/2));
 
 }
 
@@ -741,21 +985,22 @@ fill_ellipse(DiaRenderer *self,
              Color *color) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "1 1 %d %d %d %d %d 0 20 %f 1 0.0 %d %d %d %d 0 0 0 0\n",
+  fprintf(renderer->file, "1 1 %d %d %d %d %d 0 20 %s 1 0.0 %d %d %d %d 0 0 0 0\n",
 	  figLineStyle(renderer), 
 	  figLineWidth(renderer), 
 	  figColor(renderer, color),
 	  figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer),
-	  figCoord(renderer, center->x), figCoord(renderer, center->y),
-	  figCoord(renderer, width/2), figCoord(renderer, height/2));
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  (int)figCoord(renderer, center->x), (int)figCoord(renderer, center->y),
+	  (int)figCoord(renderer, width/2), (int)figCoord(renderer, height/2));
 }
 
 static void 
@@ -774,6 +1019,25 @@ draw_bezier(DiaRenderer *self,
   DIA_RENDERER_CLASS(parent_class)->draw_bezier(self, points, numpoints, color);
 }
 
+static void 
+draw_bezier_with_arrows(DiaRenderer *self, 
+			BezPoint *points,
+			int numpoints,
+			real line_width,
+			Color *color,
+			Arrow *start_arrow,
+			Arrow *end_arrow) 
+{
+  XfigRenderer *renderer = XFIG_RENDERER(self);
+
+  if (renderer->color_pass) {
+    figCheckColor(renderer, color);
+    return;
+  }
+
+  DIA_RENDERER_CLASS(parent_class)->draw_bezier_with_arrows(self, points, numpoints, line_width, color, start_arrow, end_arrow);
+}
+
 static void 
 fill_bezier(DiaRenderer *self, 
             BezPoint *points, /* Last point must be same as first point */
@@ -798,6 +1062,7 @@ draw_string(DiaRenderer *self,
 {
   guchar *figtext = NULL;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
@@ -805,14 +1070,15 @@ draw_string(DiaRenderer *self,
   }
 
   figtext = figText(renderer, text);
-  fprintf(renderer->file, "4 %d %d %d 0 %d %f 0.0 4 0.0 0.0 %d %d %s\\001\n",
+  /* xfig texts are specials */
+  fprintf(renderer->file, "4 %d %d %d 0 %d %s 0.0 6 0.0 0.0 %d %d %s\\001\n",
 	  figAlignment(renderer, alignment),
 	  figColor(renderer, color), 
 	  figDepth(renderer),
 	  figFont(renderer),
-	  figFontSize(renderer),
-	  figCoord(renderer, pos->x),
-	  figCoord(renderer, pos->y),
+	  xfig_dtostr(d_buf, figFontSize(renderer)),
+	  (int)figCoord(renderer, pos->x),
+	  (int)figCoord(renderer, pos->y),
 	  figtext);
   g_free(figtext);
 }
@@ -824,26 +1090,28 @@ draw_image(DiaRenderer *self,
            DiaImage image) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass)
     return;
 
-  fprintf(renderer->file, "2 5 %d 0 -1 0 %d 0 -1 %f %d %d 0 0 0 5\n",
+  fprintf(renderer->file, "2 5 %d 0 -1 0 %d 0 -1 %s %d %d 0 0 0 5\n",
 	  figLineStyle(renderer), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer), 
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer), 
 	  figCapsStyle(renderer));
   fprintf(renderer->file, "\t0 %s\n", dia_image_filename(image));
   fprintf(renderer->file, "\t%d %d %d %d %d %d %d %d %d %d\n",
-	  figCoord(renderer, point->x), figCoord(renderer, point->y), 
-	  figCoord(renderer, point->x+width), figCoord(renderer, point->y), 
-	  figCoord(renderer, point->x+width), figCoord(renderer, point->y+height), 
-	  figCoord(renderer, point->x), figCoord(renderer, point->y+height), 
-	  figCoord(renderer, point->x), figCoord(renderer, point->y));
+	  (int)figCoord(renderer, point->x), (int)figCoord(renderer, point->y), 
+	  (int)figCoord(renderer, point->x+width), (int)figCoord(renderer, point->y), 
+	  (int)figCoord(renderer, point->x+width), (int)figCoord(renderer, point->y+height), 
+	  (int)figCoord(renderer, point->x), (int)figCoord(renderer, point->y+height), 
+	  (int)figCoord(renderer, point->x), (int)figCoord(renderer, point->y));
 }
 
 static void 
 draw_object(DiaRenderer *self,
-            Object *object) 
+            DiaObject *object) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
 
@@ -862,11 +1130,13 @@ export_fig(DiagramData *data, const gchar *filename,
   XfigRenderer *renderer;
   int i;
   Layer *layer;
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
-  file = fopen(filename, "w");
+  file = g_fopen(filename, "w");
 
   if (file == NULL) {
-    message_error(_("Couldn't open: '%s' for writing.\n"), filename);
+    message_error(_("Can't open output file %s: %s\n"), 
+		  dia_message_filename(filename), strerror(errno));
     return;
   }
 
@@ -879,7 +1149,7 @@ export_fig(DiagramData *data, const gchar *filename,
   fprintf(file, "Center\n");
   fprintf(file, "Metric\n");
   fprintf(file, "%s\n", data->paper.name);
-  fprintf(file, "%f\n", data->paper.scaling*100.0);
+  fprintf(file, "%s\n", xfig_dtostr(d_buf, data->paper.scaling*100.0));
   fprintf(file, "Single\n"); /* Could we do layers this way? */
   fprintf(file, "-2\n");
   fprintf(file, "1200 2\n");
